/*
 * VARiD: A Variation Detection Framework for Colorspace and Letterspace platforms
 * Contact: <varid at cs dot toronto dot edu>
 * Copyright 2010 the authors (see README).
 *
 * This file is part of VARiD.
 *
 * VARiD is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * VARiD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VARiD.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>
#include <math.h>
#include "va_assert.h"
#include "va_log.h"
#include "va_time.h"
#include "varid.h"

/******************************************************************************
* get_pred generates a prediction from the given data.
*   [pred1_whole, pred2_whole] = GET_PRED(covmap, maps, ch_list, ...
*      thr_p, maxvals_2, lmat, hap1, hap2, ref)
*      generates a haplome prediction from the parameters. It does this by
*      cutting off the final probability by the given thr_p threshold, and
*      applying a few heuristics. To learn more about these heuristics see
*      the code or the paper. 
*
*   Parameters
*   covmap - the coverage maps computed during the VARID run. See
*   VARID_covmap for more details
*   maps - a struct holding states to letter or states to color maps.
*       maps.full_letter - 1600x4 matrix, the letters of each of the 1600
*           states for both alleles. For example, the second state will be
*           aa/ac, which will be represented in ns as [0 0 0 1]
*       maps.call_letter - 1600x2 matrix, only the call letter of each of
*           the states for both alleles. For example, the second state will
*           be aa/ac, with the call letters a/c, which will be represented
*           in ns as [0 1]
*       maps.call_color - 1600x2 matrix, reprenting the call colors of each
*           of the states for both alleles. For example, the second state
*           will be aa/ac, which is 0/1 in colorspace, which will be
*           represented as [0 1]
*   ch_list - size_thr x gen_len indicating destroyed posiotions at a
*       specific treshold
*   thr_p - threshold for p
*   maxvals_2 - max values in p somehow...
*   lmat - letter matrix of the best scoring alleles
*   hap1 - the haplome 1 if existent
*   hap2 - the haplome 2 if existent
*
*   Returns 
*
*   Compute the prediction for the first and second alleles given the
*   probability matrices specified. 
*******************************************************************************/
void get_pred(
    Covmaps_t      covmap, 
    Maps_t         maps, 
    VA_MatU8       ch_list,
    VA_MatDouble   maxvals_2, 
    VA_MatU8       lmat, 
    VA_VecU8       hap1, 
    VA_VecU8       hap2, 
    VA_VecU8     * pred1_whole,
    VA_VecU8     * pred2_whole)
{
    struct timespec ts;
    int rc;
    int i;
    int j;
    
    rc = va_GetTime(&ts);
    VA_ASSERT(rc == 0);

    /* prepare the predictions */
    *pred1_whole = va_allocVecU8(lmat.size_x);
    *pred2_whole = va_allocVecU8(lmat.size_x);
    for (i = 0; i < lmat.size_x; i++)
    {
        pred1_whole->data[i] = lmat.data[i];
        pred2_whole->data[i] = lmat.data[lmat.size_x + i];
    }
    
    /* correct SNPs in predictions: Method 1 */
    /* correct the SNPs in the predictions */
    /* go through each problematic snp */
    for (i = 0; i < ch_list.size_x; i++)
    {

        if (ch_list.data[i] <= 0)
            continue;

	// printf("destroystate: %i %i\n", i, ch_list.data[i]);
        
        if (ch_list.data[i] == 1) {
            /* bottom allele is good */

            pred1_whole->data[i] = pred2_whole->data[i];
            if ((pred1_whole->size >= i + 1) && 
                ((pred1_whole->data[i+1] == 4) || (pred2_whole->data[i+1] == 4))) {
                pred1_whole->data[i] = pred2_whole->data[i];
                pred1_whole->data[i + 1] = pred2_whole->data[i + 1];
            }
        }  else if (ch_list.data[i] == 2)  {
            /* top allele is good */
            /*
             * TODO: do this properly
             * here we perform a hack to clean several cases we notice: if
             * the top allele is good (pred1), and the pred 2 is a gap at
             * the NEXT position, we need to fix that for those positions as
             * well.
             */
           if ((pred2_whole->size >= i + 2) && (pred2_whole->data[i + 1] == 4)) {
               for (j = 0; j < 3; j++)
                   pred2_whole->data[i +j] = pred1_whole->data[i + j];
           } else {
                pred2_whole->data[i] = pred1_whole->data[i];
           }
                
        }  else if (ch_list.data[i] == 3)  {
            /* neither allele is good */
            /* 
             * Need to take this case correctly. Idea here is that we are
             * solving the case where one of the predictions is gapped. 
             * example: no support for color change RYGB/B-GB, but the third
             * position will be changed too!
             */
            
            if ((pred2_whole->size > i + 2) && (pred2_whole->data[i + 1] == 4)){
                for (j = 0; j < 3; j++) {
                    pred1_whole->data[i + j] = 5; 
                    pred2_whole->data[i + j] = 5;
                }
            } else {
                pred1_whole->data[i] = 5; 
                pred2_whole->data[i] = 5; 
            }
        }
    }
    
    /* CLEAN UP and DO Y? */
   
    /* only consider SNPs that have at least one coverage on BOTH allels */
    for (i = 0; i < covmap.coverage.size_x; i++)
    {
      int is_covered = ((covmap.coverage.data[i] >= min_coverage)); //&&
//((hap1.size == 0) || ((i < hap1.size) && (hap1.data[i] != 5))) && 
	//       ((hap2.size == 0) || ((i < hap2.size) && (hap2.data[i] != 5)))) ? TRUE: FALSE; 
  
      if (!is_covered)
        {
            pred1_whole->data[i] = 5;
            pred2_whole->data[i] = 5;
        }
    }
    
    /* DONT THINK WE NEED THIS ANYMORE? */

    /* compute prediction haplomes at the given threshold p */
    
    for (i = 0; i < covmap.coverage.size_x; i++)
    {
        int cant_call = ((maxvals_2.data[i] < call_threshold) || 
                         (pred1_whole->data[i] == 5)) ? TRUE : FALSE;
  
        if (cant_call)
        {
            pred1_whole->data[i] = 5;
            pred2_whole->data[i] = 5;
        }
    }
}

