/*
 * VARiD: A Variation Detection Framework for Colorspace and Letterspace platforms
 * Contact: <varid at cs dot toronto dot edu>
 * Copyright 2010 the authors (see README).
 *
 * This file is part of VARiD.
 *
 * VARiD is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * VARiD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VARiD.  If not, see <http://www.gnu.org/licenses/>.
 */

/******************************************************************************
* These macros are intended to provide for a common information logging method
* for the software.
*
* Four(4) types of message macros are defined:
*    DEBUG - messages (which need not be errors) which are of use to the
*            developer during application debug. These messages may only be
*            produced in a "debug" build and may not be in the deliverable
*            application.
*    INFO  - messages (which are not errors) which contain information of
*            value in monitoring system behavior.
*    ERROR - messages which indicate a problematic situation.
*    FATAL - application suicide note.
*
*******************************************************************************/
#ifndef VA_LOG_H
#define VA_LOG_H

#include <libgen.h>
#include <stdlib.h>
#include <syslog.h>

/* Determine delimiter to use for file (path) names */
#ifdef WIN32
#  define _D '\\'
#else
#  define _D '/'
#endif

/* Determine syslog options */
#define _LOG_OPTION   (LOG_PID | LOG_CONS)

/* Determine the default log priority level */
#ifndef NDEBUG
#  define _LOG_PRIORITY   (LOG_DEBUG)
#else
#  define _LOG_PRIORITY   (LOG_INFO)
#endif


/* Establish a log ident string */
#define LOG_IDENT(_id)                                                     \
   (((_id) != NULL) ? (_id) : ("unknown"))

#define LOG_OPEN(_ident)                                                   \
{                                                                          \
   openlog(LOG_IDENT(_ident), _LOG_OPTION, LOG_LOCAL0);                    \
   (void)setlogmask(LOG_UPTO(_LOG_PRIORITY));                              \
}


#ifndef NDEBUG

#define _LOG_(_pri_, _fmt_, _arg1_, _arg2_, _arg3_, _arg4_, _arg5_)     \
    printf(                                                             \
           " <%s:%d> " _fmt_ "\n",                                      \
            basename(__FILE__),                                         \
            __LINE__,                                                   \
           (_arg1_),                                                    \
           (_arg2_),                                                    \
           (_arg3_),                                                    \
           (_arg4_),                                                    \
           (_arg5_))

#else

#define _LOG_(_pri_, _fmt_, _arg1_, _arg2_, _arg3_, _arg4_, _arg5_)     \
    syslog((_pri_),                                                     \
           " <%s>" _fmt_,                                               \
            __FUNCTION__,                                               \
            (_arg1_),                                                   \
            (_arg2_),                                                   \
            (_arg3_),                                                   \
            (_arg4_),                                                   \
            (_arg5_))

#endif

/* Define macros to produce syslog "info" messages */
#define LOG_INFO0(_arg1_)                                                  \
   _LOG_(LOG_INFO, "%s%s%s%s%s", (_arg1_), "", "", "", "")
#define LOG_INFO1(_fmt_, _arg1_)                                           \
   _LOG_(LOG_INFO, _fmt_ "%s%s%s%s", (_arg1_), "", "", "", "")
#define LOG_INFO2(_fmt_, _arg1_, _arg2_)                                   \
   _LOG_(LOG_INFO, _fmt_ "%s%s%s", (_arg1_), (_arg2_), "", "", "")
#define LOG_INFO3(_fmt_, _arg1_, _arg2_, _arg3_)                           \
   _LOG_(LOG_INFO, _fmt_ "%s%s", (_arg1_), (_arg2_), (_arg3_), "", "")
#define LOG_INFO4(_fmt_, _arg1_, _arg2_, _arg3_, _arg4_)                   \
   _LOG_(LOG_INFO, _fmt_ "%s", (_arg1_), (_arg2_), (_arg3_), (_arg4_), "")
#define LOG_INFO5(_fmt_, _arg1_, _arg2_, _arg3_, _arg4_, _arg5_)           \
   _LOG_(LOG_INFO, _fmt_, (_arg1_), (_arg2_), (_arg3_), (_arg4_), (_arg5_))

/* Define macros to produce syslog "debug" messages */
#define LOG_DEBUG0(_arg1_)                                                  \
   _LOG_(LOG_DEBUG, "%s%s%s%s%s", (_arg1_), "", "", "", "")
#define LOG_DEBUG1(_fmt_, _arg1_)                                           \
   _LOG_(LOG_DEBUG, _fmt_ "%s%s%s%s", (_arg1_), "", "", "", "")
#define LOG_DEBUG2(_fmt_, _arg1_, _arg2_)                                   \
   _LOG_(LOG_DEBUG, _fmt_ "%s%s%s", (_arg1_), (_arg2_), "", "", "")
#define LOG_DEBUG3(_fmt_, _arg1_, _arg2_, _arg3_)                           \
   _LOG_(LOG_DEBUG, _fmt_ "%s%s", (_arg1_), (_arg2_), (_arg3_), "", "")
#define LOG_DEBUG4(_fmt_, _arg1_, _arg2_, _arg3_, _arg4_)                   \
   _LOG_(LOG_DEBUG, _fmt_ "%s", (_arg1_), (_arg2_), (_arg3_), (_arg4_), "")
#define LOG_DEBUG5(_fmt_, _arg1_, _arg2_, _arg3_, _arg4_, _arg5_)           \
   _LOG_(LOG_DEBUG, _fmt_, (_arg1_), (_arg2_), (_arg3_), (_arg4_), (_arg5_))

/* Define macros to produce syslog "error" messages */
#define LOG_ERROR0(_arg1_)                                                  \
   _LOG_(LOG_ERR, "%s%s%s%s%s", (_arg1_), "", "", "", "")
#define LOG_ERROR1(_fmt_, _arg1_)                                           \
   _LOG_(LOG_ERR, _fmt_ "%s%s%s%s", (_arg1_), "", "", "", "")
#define LOG_ERROR2(_fmt_, _arg1_, _arg2_)                                   \
   _LOG_(LOG_ERR, _fmt_ "%s%s%s", (_arg1_), (_arg2_), "", "", "")
#define LOG_ERROR3(_fmt_, _arg1_, _arg2_, _arg3_)                           \
   _LOG_(LOG_ERR, _fmt_ "%s%s", (_arg1_), (_arg2_), (_arg3_), "", "")
#define LOG_ERROR4(_fmt_, _arg1_, _arg2_, _arg3_, _arg4_)                   \
   _LOG_(LOG_ERR, _fmt_ "%s", (_arg1_), (_arg2_), (_arg3_), (_arg4_), "")
#define LOG_ERROR5(_fmt_, _arg1_, _arg2_, _arg3_, _arg4_, _arg5_)           \
   _LOG_(LOG_ERR, _fmt_, (_arg1_), (_arg2_), (_arg3_), (_arg4_), (_arg5_))

/* Define macros to produce syslog "fatal" messages */
#define LOG_FATAL0(_arg1_)                                                  \
   _LOG_(LOG_CRIT, "%s%s%s%s%s", (_arg1_), "", "", "", "")
#define LOG_FATAL1(_fmt_, _arg1_)                                           \
   _LOG_(LOG_CRIT, _fmt_ "%s%s%s%s", (_arg1_), "", "", "", "")
#define LOG_FATAL2(_fmt_, _arg1_, _arg2_)                                   \
   _LOG_(LOG_CRIT, _fmt_ "%s%s%s", (_arg1_), (_arg2_), "", "", "")
#define LOG_FATAL3(_fmt_, _arg1_, _arg2_, _arg3_)                           \
   _LOG_(LOG_CRIT, _fmt_ "%s%s", (_arg1_), (_arg2_), (_arg3_), "", "")
#define LOG_FATAL4(_fmt_, _arg1_, _arg2_, _arg3_, _arg4_)                   \
   _LOG_(LOG_CRIT, _fmt_ "%s", (_arg1_), (_arg2_), (_arg3_), (_arg4_), "")
#define LOG_FATAL5(_fmt_, _arg1_, _arg2_, _arg3_, _arg4_, _arg5_)           \
   _LOG_(LOG_CRIT, _fmt_, (_arg1_), (_arg2_), (_arg3_), (_arg4_), (_arg5_))

/* Define a macro to close the log */
#define LOG_CLOSE(_ident)                                                     \
{                                                                             \
   LOG_INFO1("%s exited", LOG_IDENT(_ident));                                 \
   closelog();                                                                \
}

#endif  /* VA_LOG_H */
