/*
 * VARiD: A Variation Detection Framework for Colorspace and Letterspace platforms
 * Contact: <varid at cs dot toronto dot edu>
 * Copyright 2010 the authors (see README).
 *
 * This file is part of VARiD.
 *
 * VARiD is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * VARiD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VARiD.  If not, see <http://www.gnu.org/licenses/>.
 */

/*******************************************************************************
*                          Description
*                          -----------
* Include inlined timing help functions
*
*******************************************************************************/
#ifndef VA_TIME_H
#define VA_TIME_H

#include <time.h>
#include <sys/timeb.h>
#include <inline.h>

#define NS_IN_SEC (1000000000) /* Number of nanoseconds in a second. */
#define NS_IN_MS  (1000000)    /* Number of nanoseconds in a millisecond. */

#define MS_IN_SEC   (1000) /* Number of milliseconds in a second. */
#define SEC_IN_MIN  (60)   /* Number of seconds in a minute. */
#define SEC_IN_HR   (3600) /* Number of seconds in an hour. */
#define MIN_IN_HR   (60)   /* Number of minutes in an hour. */
#define HR_IN_DAY   (24)   /* Number of hours in a day. */
#define MON_IN_YR   (12)   /* Number of months in a year. */

/* Define some time related constants  */
#define TIME_NSEC_PER_MSEC   (NS_IN_SEC / MS_IN_SEC)
long va_GetTime(struct timespec *tp);

/* Declare and define some 'timespec' arithmetic functions */
__EXT_INLINE (struct timespec) vaTimeAdd(
    const struct timespec * augPtr,
    const struct timespec * addPtr);

__EXT_INLINE (struct timespec) va_TimeSub(
    const struct timespec * endPtr,
    const struct timespec * startPtr);

__EXT_INLINE (int)  common_TimeEq(
    const struct timespec * t1Ptr,
    const struct timespec * t2Ptr);

__EXT_INLINE (int)  common_TimeGt(
    const struct timespec * t1Ptr,
    const struct timespec * t2Ptr);

__EXT_INLINE (int)  common_TimeLt(
    const struct timespec * t1Ptr,
    const struct timespec * t2Ptr);

#if (defined(__GNUC__) && defined(__OPTIMIZE__)) || \
    defined(__IMPLEMENT_INLINES__)

__IMP_INLINE (struct timespec) va_TimeAdd(
    const struct timespec * augPtr,
    const struct timespec * addPtr)
{
    struct timespec sum = *augPtr;

    sum.tv_sec  += addPtr->tv_sec;
    sum.tv_nsec += addPtr->tv_nsec;

    if (sum.tv_nsec >= NS_IN_SEC)
    {
        sum.tv_sec++;
        sum.tv_nsec -= NS_IN_SEC;
    }

    return sum;
}

__IMP_INLINE (struct timespec) va_TimeSub(
    const struct timespec * minPtr,
    const struct timespec * subPtr)
{
    struct timespec dif = *minPtr;

    if (dif.tv_nsec < subPtr->tv_nsec)
    {
        dif.tv_sec--;
        dif.tv_nsec += NS_IN_SEC;
    }

    dif.tv_sec  -= subPtr->tv_sec;
    dif.tv_nsec -= subPtr->tv_nsec;

    return dif;
}

__IMP_INLINE (int) va_TimeEq(
    const struct timespec * t1Ptr,
    const struct timespec * t2Ptr)
{
    int rc = 0;

    if ((t1Ptr->tv_sec == t2Ptr->tv_sec) 
        && (t1Ptr->tv_nsec == t2Ptr->tv_nsec))
    {
        rc = 1;
    }
    return (rc);
}

__IMP_INLINE (int) va_TimeGt(
    const struct timespec * t1Ptr,
    const struct timespec * t2Ptr)
{
    int rc = 0;

    if (t1Ptr->tv_sec > t2Ptr->tv_sec)
        rc = 1;
    else if (t1Ptr->tv_sec == t2Ptr->tv_sec)
        rc = (t1Ptr->tv_nsec > t2Ptr->tv_nsec) ? 1 : 0;

    return (rc);
}

__IMP_INLINE (int) va_TimeLt(
    const struct timespec * t1Ptr,
    const struct timespec * t2Ptr)
{
    int rc = 0;

    if (t1Ptr->tv_sec < t2Ptr->tv_sec)
        rc = 1;
    else if (t1Ptr->tv_sec == t2Ptr->tv_sec)
        rc = (t1Ptr->tv_nsec < t2Ptr->tv_nsec) ? 1 : 0;

    return (rc);
}

#endif /* __IMPLEMENT_INLINES__ */

#endif   /* VA_TIME_H */

